/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of Nordic
 * Semiconductor ASA.Terms and conditions of usage are described in detail
 * in NORDIC SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
 * @brief RTC/PPI/ADC modules example
 *
 * This example enables the RTC with a TICK frequency of 10Hz. Each RTC TICK triggers
 * ADC conversion through a PPI channel. Input voltage to the ADC is on AIN6 pin. Result 
 * from 8-bit sampling is displayed on port 1 (pins 8-15).
 *
 * LPCOMP is also enabled at the same time. Input voltage is on AIN4 pin. Result is
 * displayed on port 2 (pins 16-23).
 * When the voltage goes above the specified reference voltage, pin 18 (LED2) is set.
 * When the voltage goes below the specified reference voltage, pin 18 (LED2) is cleared.
 * When the voltage on AIN6 either goes above or below the specified reference voltage, 
 * pin 19 (LED3) is toggled to show that the LPCOMP interrupt handler is executed.
 *
 * @image html example_board_setup_a.png "Use board setup A for this example."
 */

#include <stdbool.h>
#include "nrf.h"
#include "nrf_delay.h"
#include "nrf_gpio.h"

#define GPIO_TOGGLE_TICK_EVENT    (8)                       /*!< Pin number to toggle when there is a tick event in RTC */
#define LFCLK_FREQUENCY           (32768UL)                 /*!< LFCLK frequency in Hertz, constant */
#define RTC_FREQUENCY             (10UL)                     /*!< required RTC working clock RTC_FREQUENCY Hertz. Changable */
#define COUNTER_PRESCALER         ((LFCLK_FREQUENCY/RTC_FREQUENCY) - 1)  /* f = LFCLK/(prescaler + 1) */

#define EVENT_TRIGGERED_PIN 	18
#define RESULT_PIN						19

/** Configures port 1 as output
 */
static void gpio_config(void)
{
	nrf_gpio_range_cfg_output(8, 15);
	nrf_gpio_range_cfg_output(16, 23);
  nrf_gpio_port_write(NRF_GPIO_PORT_SELECT_PORT1, 0);
  nrf_gpio_port_write(NRF_GPIO_PORT_SELECT_PORT2, 0);
}

static void lfclk_config(void)
{
  NRF_CLOCK->LFCLKSRC = (CLOCK_LFCLKSRC_SRC_RC << CLOCK_LFCLKSRC_SRC_Pos);
  NRF_CLOCK->EVENTS_LFCLKSTARTED = 0;
  NRF_CLOCK->TASKS_LFCLKSTART = 1;
	
	// Wait for the low frequency clock to start
  while (NRF_CLOCK->EVENTS_LFCLKSTARTED == 0) {}
  NRF_CLOCK->EVENTS_LFCLKSTARTED = 0;
}

/** Configure and start RTC
 */
static void rtc_config(void)
{
  NRF_RTC0->PRESCALER = COUNTER_PRESCALER;                   // Set prescaler to a TICK of RTC_FREQUENCY
  NRF_RTC0->EVTENSET = RTC_EVTENSET_TICK_Msk;                // Enable TICK event	
	NRF_RTC0->TASKS_START = 1;                                 // Start RTC0
}

void timer_init(void)
{		
  NRF_TIMER2->MODE = TIMER_MODE_MODE_Timer;  // Set the timer in TIMER Mode
  NRF_TIMER2->TASKS_CLEAR = 1;               // clear the task first to be usable for later
	NRF_TIMER2->PRESCALER = 4;
	NRF_TIMER2->BITMODE = TIMER_BITMODE_BITMODE_16Bit;									//Set counter to 8 bit resolution
	NRF_TIMER2->CC[0] = 25000;
	NRF_TIMER2->CC[1] = 5;
		
//  // Enable interrupt on Timer 2
//	NRF_TIMER2->INTENSET = (TIMER_INTENSET_COMPARE0_Enabled << TIMER_INTENSET_COMPARE0_Pos) | (TIMER_INTENSET_COMPARE1_Enabled << TIMER_INTENSET_COMPARE1_Pos);
//  NVIC_EnableIRQ(TIMER2_IRQn);
		
  NRF_TIMER2->TASKS_START = 1;               // Start timer
}

static void ppi_init(void)
{
  // Configure PPI channel 0 to start ADC task
  NRF_PPI->CH[0].EEP = (uint32_t)&NRF_RTC0->EVENTS_TICK;
  NRF_PPI->CH[0].TEP = (uint32_t)&NRF_ADC->TASKS_START;

  // Enable PPI channel 0
  NRF_PPI->CHEN = (PPI_CHEN_CH0_Enabled << PPI_CHEN_CH0_Pos);
}

/** Configures and enables the ADC
 */
void ADC_init(void)
{	
	/* Enable interrupt on ADC sample ready event*/		
	NRF_ADC->INTENSET = ADC_INTENSET_END_Msk;
	NVIC_EnableIRQ(ADC_IRQn);	
	
	// config ADC
	NRF_ADC->CONFIG	= (ADC_CONFIG_EXTREFSEL_None << ADC_CONFIG_EXTREFSEL_Pos) /* Bits 17..16 : ADC external reference pin selection. */
										| (ADC_CONFIG_PSEL_Disabled << ADC_CONFIG_PSEL_Pos)					/*!< Use analog input 0 as analog input. */
										| (ADC_CONFIG_REFSEL_VBG << ADC_CONFIG_REFSEL_Pos)							/*!< Use internal 1.2V bandgap voltage as reference for conversion. */
										| (ADC_CONFIG_INPSEL_SupplyOneThirdPrescaling << ADC_CONFIG_INPSEL_Pos) /*!< Analog input specified by PSEL with no prescaling used as input for the conversion. */
										| (ADC_CONFIG_RES_8bit << ADC_CONFIG_RES_Pos);									/*!< 8bit ADC resolution. */ 	
	
	
	/* Enable ADC*/
	NRF_ADC->ENABLE = ADC_ENABLE_ENABLE_Enabled;	
}

/* Interrupt handler for ADC data ready event */
void ADC_IRQHandler(void)
{
	/* Clear dataready event */
  NRF_ADC->EVENTS_END = 0;	
	
	nrf_gpio_pin_toggle(16);  //Just toggling this pin to see the actual ADC sampling frequency

  /* Write ADC result to port 1 */
 	nrf_gpio_port_write(NRF_GPIO_PORT_SELECT_PORT1, NRF_ADC->RESULT); 
	
	//Use the STOP task to save current. Workaround for PAN_028 rev1.2 anomaly 1.
  NRF_ADC->TASKS_STOP = 1;
	
	//Stop the external crystal
	NRF_CLOCK->TASKS_HFCLKSTOP = 1;
}

/** Configures and enables the LPCOMP
 */
void LPCOMP_init(void)
{	
	/* Enable interrupt on LPCOMP CROSS event */		
	NRF_LPCOMP->INTENSET = LPCOMP_INTENSET_CROSS_Msk;
	NVIC_EnableIRQ(LPCOMP_IRQn);	
	
	/* Configure LPCOMP - set input source to AVDD*4/8 */
	NRF_LPCOMP->REFSEL |= (LPCOMP_REFSEL_REFSEL_SupplyFourEighthsPrescaling << LPCOMP_REFSEL_REFSEL_Pos);
	/* Configure LPCOMP - set reference input source to AIN pin 4, i.e. P0.3 */
	NRF_LPCOMP->PSEL |= (LPCOMP_PSEL_PSEL_AnalogInput4 << LPCOMP_PSEL_PSEL_Pos);
	
	/* Enable and start the low power comparator */
	NRF_LPCOMP->ENABLE = LPCOMP_ENABLE_ENABLE_Enabled;	
	NRF_LPCOMP->TASKS_START = 1;
}



/* Interrupt handler for LPCOMP */
void LPCOMP_IRQHandler(void)
{
	// Clear event
	NRF_LPCOMP->EVENTS_CROSS = 0;
	
	// Sample the LPCOMP. Stores its state in the RESULT register. 
	// RESULT==0 means lower than reference voltage, 
	// RESULT==1 means higher than reference voltage
	NRF_LPCOMP->TASKS_SAMPLE = 1;
	nrf_gpio_pin_write(RESULT_PIN, NRF_LPCOMP->RESULT);
	
	// Toggle pin to indicate triggering of event
	nrf_gpio_pin_toggle(EVENT_TRIGGERED_PIN);
}


/**
 * main function
 */
int main(void)
{
	/* Start 16 MHz crystal oscillator */
  NRF_CLOCK->EVENTS_HFCLKSTARTED = 0;
  NRF_CLOCK->TASKS_HFCLKSTART = 1;

  /* Wait for the external oscillator to start up */
  while (NRF_CLOCK->EVENTS_HFCLKSTARTED == 0) 
  {
  }
	
  gpio_config();
	lfclk_config();
	//timer_init();
  rtc_config();
	ppi_init();
	ADC_init();
	LPCOMP_init();
	
  while (true)
  {	
		//Put CPU to sleep while waiting for interrupt
		__WFI();
  }
}

/**
 *@}
 **/
